//alexpierre.c

#include <stdio.h>
#include <stdlib.h>
#include "alexpierre.h"

#define MAXINT 0x7fffffff //#include <values.h> //marche pas dans VC++6

//----------FONCTIONS UTILITAIRES COMMUNES----------

/**
 * Retourne le minimum de deux entiers
 */
int mini(int x1, int x2)
{
  return (x1>=x2) ? x2 : x1;
}

/**
 * Retourne le maximum de deux entiers
 */
int maxi(int x1, int x2)
{
  return (x1<=x2) ? x2 : x1;
}

/**
 * Retourne dans m1 le minimum de (x0,x1) et dans m2 le maximum de (x1,x2)
 */
void miniMaxi(int x1, int x2, int *m1, int *m2)
{
  if (x1<=x2)
  {
    *m1 = x1;
    *m2 = x2;
  }
  else
  {
    *m1 = x2;
    *m2 = x1;
  }
}

/**
 * Retoune le minimum suprieur  0 de quatre entiers
 */
int mini4sup0(int x0, int x1, int x2, int x3)
{
  int result;
  if (x0 == 0) x0 = MAXINT;
  if (x1 == 0) x1 = MAXINT;
  if (x2 == 0) x2 = MAXINT;
  if (x3 == 0) x3 = MAXINT;
  result = mini(x0,mini(x1,mini(x2,x3)));
  return (result >= MAXINT) ? 0 : result;
}

//----------FONCTIONS DE TRAITEMENT D'IMAGE-------

/**
 * Retourne une chane de caractres dcrivant ce type d'image.
 */
char *ImTypeToStr(ImageType it)
{
  switch (it)
  {
    case 0 : return "binaire";
    case 1 : return "niveaux de gris";
    case 2 : return "RVB 24 bits";
    default : return "inconnu";
  }
}

/**
 * Retourne une chane de caractres dcrivant le type de cette image.
 */
char *ImTypeStr(Image im)
{
  return ImTypeToStr(ImType(im));
}

/**
 * Affiche sur la sortie standard l'image im (binaire ou niveaux de gris)
 */
void ImAffiche(Image im)
{
  int x,y,nbCol,nbRow;
  unsigned char **bitmap;
  unsigned char *ligne;
  nbRow = ImNbRow(im);
  nbCol = ImNbCol(im);
  bitmap = ImGetI(im);
  for (y=0; y<nbRow; y++)
  {
    ligne = bitmap[y];
    for (x=0; x<nbCol; x++)
      printf("%d ",ligne[x]);
    printf("\n");
  }
}

/**
 * Met  0 les bords de l'Image im (binaire ou niveaux de gris)
 */
void ImgClearBorder(Image im)
{
  int x,y,nbColM1,nbRowM1;
  unsigned char **bitmap;
  unsigned char *ligne;
  nbColM1 = ImNbCol(im)-1;
  nbRowM1 = ImNbRow(im)-1;
  bitmap = ImGetI(im);
  ligne = bitmap[0];
  for (x=nbColM1; x>=0; x--) ligne[x] = 0;
  ligne = bitmap[nbRowM1];
  for (x=nbColM1; x>=0; x--) ligne[x] = 0;
  for (y=nbRowM1-1; y>=1; y--)
  {
    ligne = bitmap[y];
    ligne[0] = 0;
    ligne[nbColM1] = 0;
  }
}

/**
 * Met  zro les pixels de im2 identiques  epsilon prs  im1
 * im1 et im2 sont deux images couleur
 */
void ImgClearIdemColor(Image im1, Image im2, int epsilon)
{
  int x,y,nbColM1;
  unsigned char **bitmapR, **bitmapV, **bitmapB, **bitmap2R, **bitmap2V, **bitmap2B;
  unsigned char *ligneR, *ligneV, *ligneB, *ligne2R, *ligne2V, *ligne2B;
  int diff;
  bitmapR = ImGetR(im1);
  bitmapV = ImGetG(im1);
  bitmapB = ImGetB(im1);
  bitmap2R = ImGetR(im2);
  bitmap2V = ImGetG(im2);
  bitmap2B = ImGetB(im2);
  nbColM1 = ImNbCol(im1)-1;
  for (y=ImNbRow(im1)-1; y>=0; y--)
  {
    ligneR = bitmapR[y];
    ligneV = bitmapV[y];
    ligneB = bitmapB[y];
    ligne2R = bitmap2R[y];
    ligne2V = bitmap2V[y];
    ligne2B = bitmap2B[y];
    for (x=nbColM1; x>=0; x--)
    {
      diff = abs(ligneR[x]-ligne2R[x]) + abs(ligneV[x]-ligne2V[x]) + abs(ligneB[x]-ligne2B[x]);
      if (diff <= epsilon)
      {
        ligne2R[x] = 0;
        ligne2V[x] = 0;
        ligne2B[x] = 0;
      }
    }
  }
}

/**
 * Retourne dans im2 les pixels de im2 s'ils sont > 0, sinon ceux de im1
 */
void ImgFusionColor(Image im1, Image im2)
{
  int x,y,nbColM1;
  unsigned char **bitmapR, **bitmapV, **bitmapB, **bitmap2R, **bitmap2V, **bitmap2B;
  unsigned char *ligneR, *ligneV, *ligneB, *ligne2R, *ligne2V, *ligne2B;
  int val;
  bitmapR = ImGetR(im1);
  bitmapV = ImGetG(im1);
  bitmapB = ImGetB(im1);
  bitmap2R = ImGetR(im2);
  bitmap2V = ImGetG(im2);
  bitmap2B = ImGetB(im2);
  nbColM1 = ImNbCol(im1)-1;
  for (y=ImNbRow(im1)-1; y>=0; y--)
  {
    ligneR = bitmapR[y];
    ligneV = bitmapV[y];
    ligneB = bitmapB[y];
    ligne2R = bitmap2R[y];
    ligne2V = bitmap2V[y];
    ligne2B = bitmap2B[y];
    for (x=nbColM1; x>=0; x--)
    {
      val = ligne2R[x]+ligne2V[x]+ligne2B[x];
      if (val == 0)
      {
        ligne2R[x] = ligneR[x];
        ligne2V[x] = ligneV[x];
        ligne2B[x] = ligneB[x];
      }
    }
  }
}

/**
 * Retourne vrai si im1 et im2 (binaires ou niveaux de gris) sont deux Image identiques, faut sinon.
 */
bool ImgEqual(Image im1, Image im2)
{
  int x,y,nbColM1;
  unsigned char **bitmap, **bitmap2;
  unsigned char *ligne1, *ligne2;
  bitmap = ImGetI(im1);
  bitmap2 = ImGetI(im2);
  nbColM1 = ImNbCol(im1)-1;
  for (y=ImNbRow(im1)-1; y>=0; y--)
  {
    ligne1 = bitmap[y];
    ligne2 = bitmap2[y];
    for (x=nbColM1; x>=0; x--)
      if (ligne1[x] != ligne2[x]) return false;
  }
  return true;
}

/**
 * Inverse logique (non logique) de cette image.
 */
void ImgNot(Image im)
{
  int x,y,nbColM1;
  unsigned char **bitmap;
  unsigned char *ligne;
  nbColM1 = ImNbCol(im)-1;
  bitmap = ImGetI(im);
  for (y=ImNbRow(im)-1; y>=0; y--)
  {
    ligne = bitmap[y];
    for (x=nbColM1; x>=0; x--)
      ligne[x] = ligne[x]==0;
  }
}

/**
 * Inverse binaire (non binaire) de cette image.
 */
void ImgBitNot(Image im)
{
  int x,y,nbColM1;
  unsigned char **bitmap;
  unsigned char *ligne;
  nbColM1 = ImNbCol(im)-1;
  bitmap = ImGetI(im);
  for (y=ImNbRow(im)-1; y>=0; y--)
  {
    ligne = bitmap[y];
    for (x=nbColM1; x>=0; x--)
      ligne[x] = !ligne[x];
  }
}

/**
 * Et logique de cette image avec la seconde.
 * La premire image est modifie.
 * Pas de vrification des dimensions des images.
 * Utilise les dimensions de l'image im1.
 */
void ImgAnd(Image im1, Image im2)
{
  int x,y,nbColM1;
  unsigned char **bitmap, **bitmap2;
  unsigned char *ligne1, *ligne2;
  bitmap = ImGetI(im1);
  bitmap2 = ImGetI(im2);
  nbColM1 = ImNbCol(im1)-1;
  for (y=ImNbRow(im1)-1; y>=0; y--)
  {
    ligne1 = bitmap[y];
    ligne2 = bitmap2[y];
    for (x=nbColM1; x>=0; x--)
      ligne1[x] = ligne1[x] && ligne2[x];
  }
}

/**
 * Et binaire de cette image avec la seconde.
 * La premire image est modifie.
 * Pas de vrification des dimensions des images.
 * Utilise les dimensions de l'image im1.
 */
void ImgBitAnd(Image im1, Image im2)
{
  int x,y,nbColM1;
  unsigned char **bitmap, **bitmap2;
  unsigned char *ligne1, *ligne2;
  bitmap = ImGetI(im1);
  bitmap2 = ImGetI(im2);
  nbColM1 = ImNbCol(im1)-1;
  for (y=ImNbRow(im1)-1; y>=0; y--)
  {
    ligne1 = bitmap[y];
    ligne2 = bitmap2[y];
    for (x=nbColM1; x>=0; x--)
      ligne1[x] = ligne1[x] && ligne2[x];
  }
}

/**
 * Ou logique de cette image avec la seconde.
 * La premire image est modifie.
 * Pas de vrification des dimensions des images.
 * Utilise les dimensions de l'image im1.
 */
void ImgOr(Image im1, Image im2)
{
  int x,y,nbColM1;
  unsigned char **bitmap, **bitmap2;
  unsigned char *ligne1, *ligne2;
  bitmap = ImGetI(im1);
  bitmap2 = ImGetI(im2);
  nbColM1 = ImNbCol(im1)-1;
  for (y=ImNbRow(im1)-1; y>=0; y--)
  {
    ligne1 = bitmap[y];
    ligne2 = bitmap2[y];
    for (x=nbColM1; x>=0; x--)
      ligne1[x] = ligne1[x] || ligne2[x];
  }
}

/**
 * Ou binaire de cette image avec la seconde.
 * La premire image est modifie.
 * Pas de vrification des dimensions des images.
 * Utilise les dimensions de l'image im1.
 */
void ImgBitOr(Image im1, Image im2)
{
  int x,y,nbColM1;
  unsigned char **bitmap, **bitmap2;
  unsigned char *ligne1, *ligne2;
  bitmap = ImGetI(im1);
  bitmap2 = ImGetI(im2);
  nbColM1 = ImNbCol(im1)-1;
  for (y=ImNbRow(im1)-1; y>=0; y--)
  {
    ligne1 = bitmap[y];
    ligne2 = bitmap2[y];
    for (x=nbColM1; x>=0; x--)
      ligne1[x] = ligne1[x] | ligne2[x];
  }
}

/**
 * Fusionne les labels de deux pixels voisins
 * Pour ImgCountObjects et ImgLabelliseObjects
 */
int _fusionLabels(int *labels, int x, int y)
{
  int m1, m2, t;
  if (x == y) return labels[x];
  else
  {
    miniMaxi(x,y,&m1,&m2); //m1 = mini ; m2 = maxi
    t = labels[m2];
    labels[m2] = m1; //il faut mettre  jour l'information ds maintenant pour viter les boucles
    return labels[m2] = _fusionLabels(labels,m1,t);
  }
}

/**
 * Compte le nombre d'objets prsents dans l'image im (binaire ou niveau de gris)
 */
int ImgCountObjects(Image im)
{
  int x,y,nbCol,nbRow,tmp,tmp2,
      maxlabel, //numro max de label
      compteur; //compteur de numro de label
  int *labels; //tableau des labels
  unsigned char **bitmap; //raccourci vers im (Image)
  unsigned char *lC; //ligne Centre
  Matrix mt1; //image des labels
  int **bitmap2; //raccourci vers mt1 (Matrice)
  int *lRN, //ligne au nord du rsultat
      *lR; //ligne Rsultat

  bitmap = ImGetI(im);
  nbCol = ImNbCol(im);
  nbRow = ImNbRow(im);
  mt1 = MatAlloc(Int,nbRow,nbCol);
  bitmap2 = MatGetInt(mt1);
  maxlabel = (1+nbRow/2) * (1+nbCol/2); //nombre maximum d'objets diffrents
  labels = malloc(sizeof(int)*maxlabel);
  compteur = 0;
  labels[0] = 0; //tiquette du fond

  //1re ligne
  y = 0;
  lC = bitmap[y];
  lR = bitmap2[y];
  x = 0;
  if (lC[x]>0) //1re case
  {
    compteur++; //premier objet
    lR[x] = compteur;
    labels[compteur] = compteur;
  }
  else lR[x] = 0;

  for (x=1; x<nbCol; x++)
    if (lC[x] == 0) lR[x] = 0;
    else if (lR[x-1] == 0)
    {
      compteur++; //on incrmente compteur si on a un nouvel objet
      lR[x] = compteur;
      labels[compteur] = compteur;
    }
    else lR[x] = lR[x-1];

  //les autres lignes
  for (y=1; y<nbRow; y++)
  {
    lC = bitmap[y];
    lRN = bitmap2[y-1];
    lR = bitmap2[y];
    for (x=0; x<nbCol; x++)
    {
      if (lC[x] == 0) lR[x] = 0;
      else
      {
        tmp = mini4sup0(((x>0) ? lR[x-1] : 0), ((x>0) ? lRN[x-1] : 0), lRN[x], ((x<nbCol-1) ? lRN[x+1] : 0)); //regarde les voisins
        if (tmp == 0) //nouvel objet
        {
          compteur++; //on incrmente compteur
          lR[x] = compteur;
          labels[compteur] = compteur;
        }
        else //pixel connect : gre les connections et les fusions possibles
        {
          if ((x>0) && ((tmp2=lR[x-1]) != tmp))
          {
            if (tmp2 > 0) tmp = _fusionLabels(labels,tmp,tmp2); //avec le pixel de gauche
            else if (((tmp2=lRN[x-1]) != tmp) && (tmp2 > 0)) tmp = _fusionLabels(labels,tmp,tmp2); //sinon avec le pixel en haut  gauche
          }
          if ((x<nbCol-1) && ((tmp2=lRN[x+1]) != tmp) && (tmp2 > 0)) tmp = _fusionLabels(labels,tmp,tmp2); //avec le pixel en haut  droite
          lR[x] = tmp;
        }
      }
    }
  }
  MatFree(&mt1);
  for (x=compteur; x>0; x--) //dcompte les tiquettes multiples pour un mme objet.
    if (labels[x]!=x) compteur--;
  free(labels);
  return compteur;
}

/**
 * Labellise les diffrents objets prsents dans l'image im1 (binaire ou niveau de gris)
 * et le marque dans la matrice mt1 (cre si NULL)
 * Retourne le nombre d'objets prsents dans l'image
 */
int ImgLabelliseObjects(Image im1, Matrix mt1)
{
  int x,y,nbCol,nbRow,tmp,tmp2,
      maxlabel, //numro max de label
      compteur; //compteur de numro de label
  int *labels; //tableau des labels
  unsigned char **bitmap; //raccourci vers im1 (Image), puis im2 (Image)
  unsigned char *lC; //ligne Centre
  int **bitmap2; //raccourci vers mt1 (Matrice)
  int *lRN, //ligne au nord du rsultat
      *lR; //ligne Rsultat

  bitmap = ImGetI(im1);
  nbCol = ImNbCol(im1);
  nbRow = ImNbRow(im1);
  if (mt1 == NULL) mt1 = MatAlloc(Int,nbRow,nbCol); //cration de la matrice de sortie si elle n'existe pas
  bitmap2 = MatGetInt(mt1);
  maxlabel = (1+nbRow/2) * (1+nbCol/2); //nombre maximum d'objets diffrents
  labels = malloc(sizeof(int)*maxlabel);
  compteur = 0;
  labels[0] = 0; //tiquette du fond

  //1re ligne
  y = 0;
  lC = bitmap[y];
  lR = bitmap2[y];
  x = 0;
  if (lC[x]>0) //1re case
  {
    compteur++; //premier objet
    lR[x] = compteur;
    labels[compteur] = compteur;
  }
  else lR[x] = 0;
  for (x=1; x<nbCol; x++)
    if (lC[x] == 0) lR[x] = 0;
    else if (lR[x-1] == 0)
    {
      compteur++; //on incrmente compteur si on a un nouvel objet
      lR[x] = compteur;
      labels[compteur] = compteur;
    }
    else lR[x] = lR[x-1];

  //les autres lignes
  for (y=1; y<nbRow; y++)
  {
    lC = bitmap[y];
    lRN = bitmap2[y-1];
    lR = bitmap2[y];
    for (x=0; x<nbCol; x++)
    {
      if (lC[x] == 0) lR[x] = 0;
      else
      {
        tmp = mini4sup0(((x>0) ? lR[x-1] : 0), ((x>0) ? lRN[x-1] : 0), lRN[x], ((x<nbCol-1) ? lRN[x+1] : 0)); //regarde les voisins
        if (tmp == 0) //nouvel objet
        {
          compteur++; //on incrmente compteur
          lR[x] = compteur;
          labels[compteur] = compteur;
        }
        else //pixel connect : gre les connections et les fusions possibles
        {
          if ((x>0) && ((tmp2=lR[x-1]) != tmp))
          {
            if (tmp2 > 0) tmp = _fusionLabels(labels,tmp,tmp2); //avec le pixel de gauche
            else if (((tmp2=lRN[x-1]) != tmp) && (tmp2 > 0)) tmp = _fusionLabels(labels,tmp,tmp2); //sinon avec le pixel en haut  gauche
          }
          if ((x<nbCol-1) && ((tmp2=lRN[x+1]) != tmp) && (tmp2 > 0)) tmp = _fusionLabels(labels,tmp,tmp2); //avec le pixel en haut  droite
          lR[x] = tmp;
        }
      }
    }
  }

  tmp = compteur;
  for (x=1; x<=tmp; x++) //fusionne les tiquettes restes isoles
    if (labels[x]!=x)
    {
      labels[x] = _fusionLabels(labels,x,labels[x]);
      compteur--;
    }

  /*TODO : Minimiser les numros des labels */

  //bitmap = ImGetI(im2);
  for (y=0; y<nbRow; y++)
  {
    //lC = bitmap[y]; //image de sortie
    lR = bitmap2[y]; //matrice d'entre
    for (x=0; x<nbCol; x++)
      lR[x] = labels[lR[x]];
      //lC[x] = labels[lR[x]];
  }
  //MatFree(&mt1);
  free(labels);
  return compteur;
}

/**
 * Retourne nbColors diffrentes dans l'image ligne im1 (cre si NULL)
 */
void ImgCreateColors(int nbColors, Image im1)
{
  if (im1 == NULL) im1 = ImAlloc(Color,1,nbColors);

}

/**
 * Met en couleur la matrice de labels mt1 et la retourne dans l'image couleur im1
 */
void ImgColoriseLabels(Matrix mt1, int nbLabels, Image im1)
{

}

/**
 * Oprateur d'amincicement sur l'image im.
 */
void ImgThin(Image im)
{
  int x,y,nbColM2,nbRowM2;
  Image im2;
  unsigned char **bitmap,
                **bitmap2;
  unsigned char *lN, //ligne Nord
                *lC, //ligne Centre
                *lS, //ligne Sud
                *lR; //ligne Rsultat

  nbColM2 = ImNbCol(im)-2;
  nbRowM2 = ImNbRow(im)-2;
  bitmap = ImGetI(im);
  im2 = ImAlloc(BitMap,ImNbRow(im),ImNbCol(im));
  bitmap2 = ImGetI(im2);

  /*
  for (y=ImNbRow(im)-2; y>0; y--) //attention aux bords de l'image
  {
    lN = bitmap[y-1];
    lC = bitmap[y];
    lS = bitmap[y+1];
    lR = bitmap2[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (  (lN[x-1] && lN[x] && lN[x+1] && !(lS[x-1] || lS[x] || lS[x+1]))
              || (lN[x] && lN[x+1] && lC[x+1] && !(lC[x-1] || lS[x-1] || lS[x]))
              || (lN[x+1] && lC[x+1] && lS[x+1] && !(lN[x-1] || lC[x-1] || lS[x-1]))
              || (lC[x+1] && lS[x] && lS[x+1] && !(lN[x-1] || lN[x] || lC[x-1]))
              || (lS[x-1] && lS[x] && lS[x+1] && !(lN[x-1] || lN[x] || lN[x+1]))
              || (lC[x-1] && lS[x-1] && lS[x] && !(lN[x] || lN[x+1] || lC[x+1]))
              || (lN[x-1] && lC[x-1] && lS[x-1] && !(lN[x+1] || lC[x+1] || lS[x+1]))
              || (lN[x-1] && lN[x] && lC[x-1] && !(lC[x+1] || lS[x] || lS[x+1]))
              ) ? 0 : lC[x];
  }
  ImFree(&im);
  im = ImCopy(im2);
  */
  for (y=nbRowM2; y>0; y--) //1 //attention aux bords de l'image
  {
    lN = bitmap[y-1];
    lC = bitmap[y];
    lS = bitmap[y+1];
    lR = bitmap2[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x-1] && lN[x] && lN[x+1] && !(lS[x-1] || lS[x] || lS[x+1])) ? 0 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //2
  {
    lN = bitmap2[y-1];
    lC = bitmap2[y];
    lS = bitmap2[y+1];
    lR = bitmap[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x] && lN[x+1] && lC[x+1] && !(lC[x-1] || lS[x-1] || lS[x])) ? 0 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //3
  {
    lN = bitmap[y-1];
    lC = bitmap[y];
    lS = bitmap[y+1];
    lR = bitmap2[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x+1] && lC[x+1] && lS[x+1] && !(lN[x-1] || lC[x-1] || lS[x-1])) ? 0 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //4
  {
    lN = bitmap2[y-1];
    lC = bitmap2[y];
    lS = bitmap2[y+1];
    lR = bitmap[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lC[x+1] && lS[x] && lS[x+1] && !(lN[x-1] || lN[x] || lC[x-1])) ? 0 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //5
  {
    lN = bitmap[y-1];
    lC = bitmap[y];
    lS = bitmap[y+1];
    lR = bitmap2[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lS[x-1] && lS[x] && lS[x+1] && !(lN[x-1] || lN[x] || lN[x+1])) ? 0 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //6
  {
    lN = bitmap2[y-1];
    lC = bitmap2[y];
    lS = bitmap2[y+1];
    lR = bitmap[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lC[x-1] && lS[x-1] && lS[x] && !(lN[x] || lN[x+1] || lC[x+1])) ? 0 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //7
  {
    lN = bitmap[y-1];
    lC = bitmap[y];
    lS = bitmap[y+1];
    lR = bitmap2[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x-1] && lC[x-1] && lS[x-1] && !(lN[x+1] || lC[x+1] || lS[x+1])) ? 0 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //8
  {
    lN = bitmap2[y-1];
    lC = bitmap2[y];
    lS = bitmap2[y+1];
    lR = bitmap[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x-1] && lN[x] && lC[x-1] && !(lC[x+1] || lS[x] || lS[x+1])) ? 0 : lC[x];
  }
  ImFree(&im2);
}

/**
 * Remplit les concavits
 */
void ImgFillConcav(Image im)
{
  int x,y,nbColM2,nbRowM2;
  Image im2;
  unsigned char **bitmap,
                **bitmap2;
  unsigned char *lN, //ligne Nord
                *lC, //ligne Centre
                *lS, //ligne Sud
                *lR; //ligne Rsultat

  nbColM2 = ImNbCol(im)-2;
  nbRowM2 = ImNbRow(im)-2;
  bitmap = ImGetI(im);
  im2 = ImAlloc(BitMap,ImNbRow(im),ImNbCol(im));
  bitmap2 = ImGetI(im2);

  for (y=nbRowM2; y>0; y--) //1 //attention aux bords de l'image
  {
    lN = bitmap[y-1];
    lC = bitmap[y];
    lS = bitmap[y+1];
    lR = bitmap2[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x-1] && lC[x-1] && lC[x+1] && lS[x-1] && lS[x]) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //2
  {
    lN = bitmap2[y-1];
    lC = bitmap2[y];
    lS = bitmap2[y+1];
    lR = bitmap[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x-1] && lN[x] && lC[x-1] && lS[x-1] && lS[x+1]) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //3
  {
    lN = bitmap[y-1];
    lC = bitmap[y];
    lS = bitmap[y+1];
    lR = bitmap2[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x-1] && lN[x] && lN[x+1] && lC[x-1] && lS[x]) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //4
  {
    lN = bitmap2[y-1];
    lC = bitmap2[y];
    lS = bitmap2[y+1];
    lR = bitmap[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x-1] && lN[x] && lN[x+1] && lC[x+1] && lS[x-1]) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //5
  {
    lN = bitmap[y-1];
    lC = bitmap[y];
    lS = bitmap[y+1];
    lR = bitmap2[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x] && lN[x+1] && lC[x-1] && lC[x+1] && lS[x+1]) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //6
  {
    lN = bitmap2[y-1];
    lC = bitmap2[y];
    lS = bitmap2[y+1];
    lR = bitmap[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x-1] && lN[x+1] && lC[x+1] && lS[x] && lS[x+1]) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //7
  {
    lN = bitmap[y-1];
    lC = bitmap[y];
    lS = bitmap[y+1];
    lR = bitmap2[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x] && lC[x+1] && lS[x-1] && lS[x] && lS[x+1]) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //8
  {
    lN = bitmap2[y-1];
    lC = bitmap2[y];
    lS = bitmap2[y+1];
    lR = bitmap[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x+1] && lC[x-1] && lS[x-1] && lS[x] && lS[x+1]) ? 1 : lC[x];
  }
  ImFree(&im2);
}

/**
 * Oprateur d'paississement sur l'image im.
 */
void ImgThicken(Image im)
{
  int x,y,nbColM2,nbRowM2;
  Image im2;
  unsigned char **bitmap,
                **bitmap2;
  unsigned char *lN, //ligne Nord
                *lC, //ligne Centre
                *lS, //ligne Sud
                *lR; //ligne Rsultat

  ImgFillConcav(im); //on commence par remplir les concavits
  nbColM2 = ImNbCol(im)-2;
  nbRowM2 = ImNbRow(im)-2;
  bitmap = ImGetI(im);
  im2 = ImAlloc(BitMap,ImNbRow(im),ImNbCol(im));
  bitmap2 = ImGetI(im2);

  for (y=nbRowM2; y>0; y--) //1 //attention aux bords de l'image
  {
    lN = bitmap[y-1];
    lC = bitmap[y];
    lS = bitmap[y+1];
    lR = bitmap2[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x-1] && lN[x] && lN[x+1] && !(lS[x-1] || lS[x] || lS[x+1])) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //2
  {
    lN = bitmap2[y-1];
    lC = bitmap2[y];
    lS = bitmap2[y+1];
    lR = bitmap[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x] && lN[x+1] && lC[x+1] && !(lC[x-1] || lS[x-1] || lS[x])) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //3
  {
    lN = bitmap[y-1];
    lC = bitmap[y];
    lS = bitmap[y+1];
    lR = bitmap2[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x+1] && lC[x+1] && lS[x+1] && !(lN[x-1] || lC[x-1] || lS[x-1])) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //4
  {
    lN = bitmap2[y-1];
    lC = bitmap2[y];
    lS = bitmap2[y+1];
    lR = bitmap[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lC[x+1] && lS[x] && lS[x+1] && !(lN[x-1] || lN[x] || lC[x-1])) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //5
  {
    lN = bitmap[y-1];
    lC = bitmap[y];
    lS = bitmap[y+1];
    lR = bitmap2[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lS[x-1] && lS[x] && lS[x+1] && !(lN[x-1] || lN[x] || lN[x+1])) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //6
  {
    lN = bitmap2[y-1];
    lC = bitmap2[y];
    lS = bitmap2[y+1];
    lR = bitmap[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lC[x-1] && lS[x-1] && lS[x] && !(lN[x] || lN[x+1] || lC[x+1])) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //7
  {
    lN = bitmap[y-1];
    lC = bitmap[y];
    lS = bitmap[y+1];
    lR = bitmap2[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x-1] && lC[x-1] && lS[x-1] && !(lN[x+1] || lC[x+1] || lS[x+1])) ? 1 : lC[x];
  }
  for (y=nbRowM2; y>0; y--) //8
  {
    lN = bitmap2[y-1];
    lC = bitmap2[y];
    lS = bitmap2[y+1];
    lR = bitmap[y];
    for (x=nbColM2; x>0; x--)
      lR[x] = (lN[x-1] && lN[x] && lC[x-1] && !(lC[x+1] || lS[x] || lS[x+1])) ? 1 : lC[x];
  }
  ImFree(&im2);
}

/**
 * Oprateur de skeleton par itrations de ImgThin sur l'image im.
 */
int ImgThinSkeleton(Image im)
{
  int i;
  Image im2;
  im2 = ImAlloc(BitMap,1,1); //initialisation
  i = 0;
  do
  {
    ImFree(&im2);
    im2 = ImCopy(im);
    ImgThin(im);
    i++;
  } while (!ImgEqual(im,im2));
  ImFree(&im2);
  ImgClearBorder(im);
  return i;
}
