//alexpierre.h

#ifndef __alexpierre_h__
#define __alexpierre_h__

#include "limace.h"

#ifdef __cplusplus
  extern "C" {
#else
  typedef char bool;
  #define false 0;
  #define true 1;
#endif


//----------FONCTIONS UTILITAIRES COMMUNES----------

/**
 * Retourne le minimum de deux entiers
 */
extern int mini(int x1, int x2);

/**
 * Retourne le maximum de deux entiers
 */
extern int maxi(int x1, int x2);

/**
 * Retourne dans m1 le minimum de (x0,x1) et dans m2 le maximum de (x1,x2)
 */
extern void miniMaxi(int x1, int x2, int *m1, int *m2);

/**
 * Retoune le minimum suprieur  0 de quatre entiers
 */
extern int mini4sup0(int x0, int x1, int x2, int x3);


//----------FONCTIONS DE TRAITEMENT D'IMAGE-------

/**
 * Retourne une chane de caractres dcrivant ce type d'image.
 */
extern char *ImTypeToStr(ImageType it);

/**
 * Retourne une chane de caractres dcrivant le type de cette image.
 */
extern char *ImTypeStr(Image im);

/**
 * Affiche sur la sortie standard l'image im (binaire ou niveaux de gris)
 */
extern void ImAffiche(Image im);

/**
 * Met  0 les bords de l'Image im (binaire ou niveaux de gris)
 */
extern void ImgClearBorder(Image im);

/**
 * Met  zro les pixels de im2 identiques  epsilon prs  im1
 * im1 et im2 sont deux images couleur
 */
extern void ImgClearIdemColor(Image im1, Image im2, int epsilon);

/**
 * Retourne dans im2 les pixels de im2 s'ils sont > 0, sinon ceux de im1
 * im1 et im2 sont deux images couleur
 */
extern void ImgFusionColor(Image im1, Image im2);

/**
 * Retourne vrai si im1 et im2 (binaires ou niveaux de gris) sont deux Image identiques, faut sinon.
 */
extern bool ImgEqual(Image im1, Image im2);

/**
 * Inverse logique (non logique) de cette image.
 */
extern void ImgNot(Image im);

/**
 * Inverse binaire (non binaire) de cette image.
 */
extern void ImgBitNot(Image im);

/**
 * Et logique de cette image avec la seconde.
 * La premire image est modifie.
 * Pas de vrification des dimensions des images.
 * Utilise les dimensions de l'image Im1.
 */
extern void ImgAnd(Image im1, Image im2);

/**
 * Et binaire de cette image avec la seconde.
 * La premire image est modifie.
 * Pas de vrification des dimensions des images.
 * Utilise les dimensions de l'image Im1.
 */
void ImgBitAnd(Image im1, Image im2);

/**
 * Ou logique de cette image avec la seconde.
 * La premire image est modifie.
 * Pas de vrification des dimensions des images.
 * Utilise les dimensions de l'image Im1.
 */
extern void ImgOr(Image im1, Image im2);

/**
 * Ou binaire de cette image avec la seconde.
 * La premire image est modifie.
 * Pas de vrification des dimensions des images.
 * Utilise les dimensions de l'image Im1.
 */
extern void ImgBitOr(Image im1, Image im2);

/**
 * Compte le nombre d'objets prsents dans l'image im (binaire ou niveau de gris)
 */
extern int ImgCountObjects(Image im);

/**
 * Labellise les diffrents objets prsents dans l'image im1 (binaire ou niveau de gris)
 * et le marque dans la matrice mt1 (cre si NULL)
 * Retourne le nombre d'objets prsents dans l'image
 */
extern int ImgLabelliseObjects(Image im1, Matrix mt1);

/**
 * Met en couleur la matrice de labels mt1 et la retourne dans l'image couleur im1
 */
extern void ImgColoriseLabels(Matrix mt1, int nbLabels, Image im1);

/**
 * Oprateur d'amincicement sur l'image im.
 */
extern void ImgThin(Image im);

/**
 * Remplit les concavits
 */
extern void ImgFillConcav(Image im);

/**
 * Oprateur d'paississement sur l'image im.
 */
extern void ImgThicken(Image im);

/**
 * Oprateur de skeleton par itrations de ImgThin sur l'image im.
 * Retourne le nombre d'itrations ncessaires.
 */
extern int ImgThinSkeleton(Image im);

#ifdef __cplusplus
  }
#endif

#endif
